
class MalformedGoDiagram(ValueError): 
    pass

class GoDiagramParser(object):
    line_prefix = '$$'

    def __init__(self):
        self.top_edge = False
        self.left_edge = False
        self.right_edge = False
        self.bottom_edge = False
        self.board_width = 0
        self.board_height = 0
        self.initial_black = set()
        self.initial_white = set()
        self.sequence_moves = {}
        self.marked_coordinates = {}
        self.first_player = 'black'
        self.second_player = 'white'
        self._state = 'header'

    def board_size(self):
        return self.board_width, self.board_height

    def parse(self, board):
        for line in board:
            if self._state == 'header':
                self.parse_header(line)
            elif self._state == 'board':
                self.parse_board(line)
            elif self._state == 'footer':
                self.parse_footer(line)

    def parse_header(self, line):
        header_line = False
        line = line.strip()
        if line.startswith(self.line_prefix + 'W'):
            header_line = True
            self.first_player = 'white'
            self.second_player = 'black'
            prefix = self.line_prefix + 'W'
        elif line.startswith(self.line_prefix + 'B'):
            header_line = True
            self.first_player = 'black'
            self.second_player = 'white'
            prefix = self.line_prefix + 'B'
        elif line.startswith(self.line_prefix):
            prefix = self.line_prefix
        else:
            raise MalformedGoDiagram("All lines must start with $$")

        if '---' in line:
            header_line = True
            line = line[len(prefix):]
            line = line.strip()
            # top border:
            if any(c != '-' for c in line):
                raise MalformedGoDiagram('Top edge must consist of only "-" characters')
            self.top_edge=True
        if not header_line:
            self._state = 'board'
            self.parse_board(line)
                
    def parse_board(self, line):
        line = line.strip()
        if not line.startswith(self.line_prefix):
            raise MalformedGoDiagram("All lines must start with $$")
        line = line[len(self.line_prefix):]
        line = line.strip()
        if '---' in line:
            self._state == 'footer'
            self.parse_footer(line)
        else:
            self.board_height += 1
            if line[0] == '|':
                self.left_edge = True
                line = line[1:]
            if line[-1] == '|':
                self.right_edge = True
                line = line[:-1]
            points = line.split()
            if not self.board_width:
                self.board_width = len(points)
            elif self.board_width != len(points):
                raise MalformedGoDiagram("All lines must have the same number of points")
            for i, point in enumerate(points):
                coord = (i, self.board_height - 1)
                if point == 'X':
                    self.initial_black.add(coord)
        	    self.marked_coordinates[coord] = ('black', None)
                elif point == 'O':
                    self.initial_white.add(coord)
        	    self.marked_coordinates[coord] = ('white', None)
                elif all(c in list('1234567890') for c in point):
                    point = int(point)
                    if point == 0:
                        point = 10
                    self.sequence_moves[point] = coord
        	    if point % 2:
        	        player = self.first_player
        	    else:
        		player = self.second_player
        	    self.marked_coordinates[coord] = (player, str(point))
 
                elif point in list(u'abcdefghijklmnopqrstuvwxyz'):
		    # empty point annotations
		    self.marked_coordinates[coord] = (u'empty', point)
		elif point in list(u'BW#@YQZPCSTM'):
                    translations = {
                        'B': (u'black', u'O'),
                        'W': (u'white', u'O'), 
                        'C': (u'empty', u'O'), #MEDIUM SMALL WHITE CIRCLE, WHITE CIRCLE?
                        '#': (u'black', u'\N{BLACK MEDIUM SQUARE}'), 
                        '@': (u'white', u'\N{BLACK MEDIUM SQUARE}'),
                        'S': (u'empty', u'\N{BLACK MEDIUM SQUARE}'), # BLACK SQUARE
                        'Y': (u'black', u'\N{BLACK UP-POINTING TRIANGLE}'),
                        'Q': (u'white', u'\N{BLACK UP-POINTING TRIANGLE}'),
                        'T': (u'empty', u'\N{BLACK UP-POINTING TRIANGLE}'),
                        'Z': (u'black', u'X'),
                        'P': (u'white', u'X'),
                        'M': (u'white', u'X'),
                    }

		    # stone annotations
        	    self.marked_coordinates[coord] = translations[point]
                elif point in list('.,'):
                    #Empty point
                    pass
                else:
                    raise MalformedGoDiagram('Unrecognized board marking: "%s"' % point)
                    
    def parse_footer(self, line):
        if '---' in line:
            self.bottom_edge = True
            if any(c != '-' for c in line):
                raise MalformedGoDiagram('Bottom edge must consist of only "-" characters')
        else:
            #Not handling other footer information yet.
            pass




from __future__ import division

from itertools import cycle
import os

from PIL import Image, ImageDraw, ImageFont

FONTPATH = "/usr/share/fonts/truetype/freefont"

class TileSet(object):

    def __init__(self, name='tiny'):
        self.name = name
        self.tile_size = (14, 14)
        self.resource_dir = '%s/resources/%s' % (os.path.dirname(__file__), name)
           
        self.tile_cache = {}
        self.valid_tiles = [
            'tl-corner', 'tr-corner', 'bl-corner', 'br-corner',
            't-side', 'b-side', 'l-side', 'r-side',
            'point', 'starpoint', 'empty',
            'black', 'white',
        ]

    def __getitem__(self, tile):
        if tile not in self.valid_tiles:
            raise KeyError
        if tile not in self.tile_cache:
            self.tile_cache[tile] = Image.open('%s/%s.png' % (self.resource_dir, tile))
        return self.tile_cache[tile]

    def annotate(self, tile, annotation):

        if tile == 'black':
            fgcolor = (255, 255, 255)
        else:
            fgcolor = (0, 0, 0)
        annotated_tile_name = '-'.join([tile, annotation])
        if annotated_tile_name not in self.tile_cache:
            tile_object = self[tile].copy()
            tile_object = self.render_annotation(
                tile_object, 
                annotation, 
                fgcolor,
            )
            self.tile_cache[annotated_tile_name] = tile_object
        return self.tile_cache[annotated_tile_name]
      
    def render_annotation(self, stone, annotation, fgcolor):
        fontsize_mapping = {
            1: 11,
            2: 9,
            3: 8
        }

        fontsize = fontsize_mapping.get(len(annotation), 11)
        font = ImageFont.truetype(os.path.join(FONTPATH, 'FreeSans.ttf'), 
                                  fontsize)
        label = Image.new('RGBA', self.tile_size, (255,255,255,255))
        draw = ImageDraw.Draw(stone) 
        label_size = draw.textsize(annotation, font=font)
        offset = (
            (self.tile_size[0] - label_size[0]) // 2,
            (self.tile_size[1] - label_size[1]) // 2 + 1
        )
        draw.text(offset, annotation, fill=fgcolor, font=font)
        return stone

    def render(self, diagram):
        image = Image.new('RGBA', 
            (diagram.board.size[0] * self.tile_size[0], 
             diagram.board.size[1] * self.tile_size[1])
        )
        for x in xrange(diagram.board.size[0]):
            for y in xrange(diagram.board.size[1]):
                point = x, y
                if point in diagram.initial_black:
                    tile = 'black'
                elif point in diagram.initial_white:
                    tile = 'white'
                
                elif diagram.board.left_edge and x == 0:
                    if diagram.board.top_edge and y == 0:
                       tile = 'tl-corner'
                    elif diagram.board.bottom_edge and y == diagram.board.size[1] - 1:
                       tile = 'bl-corner'
                    else:
                       tile = 'l-side'
                elif diagram.board.right_edge and x == diagram.board.size[0] - 1:
                    if diagram.board.top_edge and y == 0:
                       tile = 'tr-corner'
                    elif diagram.board.bottom_edge and y == diagram.board.size[1] - 1:
                       tile = 'br-corner'
                    else:
                       tile = 'r-side'
                elif diagram.board.top_edge and y == 0:
                    tile='t-side'
                elif diagram.board.bottom_edge and y == diagram.board.size[1] - 1:
                    tile='b-side'
                elif (x, y) in diagram.board.star_points:
                    tile = 'starpoint'
                else:
                    tile = 'point'
                box = (x*self.tile_size[0], y*self.tile_size[1],
                       (x+1)*self.tile_size[0], (y+1)*self.tile_size[1])
                image.paste(self[tile], box)

        for c in diagram.marked_coordinates:
            x, y = c
            box = (x*self.tile_size[0], y*self.tile_size[1],
                   (x+1)*self.tile_size[0], (y+1)*self.tile_size[1])
            mark = diagram.marked_coordinates[c]
            if mark[1]:
            	tile = self.annotate(mark[0], mark[1])
            else:
                tile = mark[0]
            image.paste(tile, box)
        return image


class Board(object):

    def __init__(self, size=(19, 19), left_edge=True, right_edge=True, top_edge=True, bottom_edge=True):
        self.size=size
        self.top_edge = top_edge
        self.left_edge = left_edge
        self.right_edge = right_edge
        self.bottom_edge = bottom_edge
        self.set_star_points() 
        
    def set_star_points(self):
        x_ranks = set()
        y_ranks = set()
        if self.left_edge and not self.right_edge:
            if self.size[0] >= 4:
                x_ranks.add(3)
            if self.size[0] >= 10:
                x_ranks.add(9)
            if self.size[0] >= 16:
                x_ranks.add(15)

        elif self.right_edge and not self.left_edge:
            if self.size[0] >= 4:
                x_ranks.add(self.size[0] - 4)
            if self.size[0] >= 10:
                x_ranks.add(self.size[0] - 10)
            if self.size[0] >= 16:
                x_ranks.add(self.size[0] - 16)
            
        elif self.left_edge and self.right_edge:
            if self.size[0] >=9:
                x_ranks.add(3)
                x_ranks.add(self.size[0] - 4)
            if self.size[0] % 2 and self.size[0] >= 15:
                x_ranks.add((self.size[0] - 1) // 2)

        if self.top_edge and not self.bottom_edge:
            if self.size[1] >= 4:
                y_ranks.add(3)
            if self.size[1] >= 10:
                y_ranks.add(9)
            if self.size[1] >= 16:
                y_ranks.add(15)
 
        elif not self.top_edge and self.bottom_edge:
            if self.size[1] >= 4:
                y_ranks.add(self.size[1] - 4)
            if self.size[1] >= 10:
                y_ranks.add(self.size[1] - 10)
            if self.size[1] >= 16:
                y_ranks.add(self.size[1] - 16)

        elif self.top_edge and self.bottom_edge:
            if self.size[1] >=9:
                y_ranks.add(3)
                y_ranks.add(self.size[1] - 4)
            if self.size[1] % 2 and self.size[1] >= 15:
                y_ranks.add((self.size[1] - 1) // 2)
    
        self.star_points = set()
        for x in x_ranks:
            for y in y_ranks:
                self.star_points.add((x, y))


class GoDiagram(object):

    @classmethod
    def load_from_parser(cls, parser):
        board = Board(
            size=parser.board_size(),
            top_edge=parser.top_edge,
            left_edge=parser.left_edge,
            right_edge=parser.right_edge,
            bottom_edge=parser.bottom_edge,
        )
        return cls(board=board,
            initial_black=parser.initial_black,
            initial_white=parser.initial_white,
            marked_coordinates=parser.marked_coordinates,
            first_player=parser.first_player,
        )

    def __init__(self, board=Board(), initial_black=None, initial_white=None, marked_coordinates=None, first_player='black'):
        self.board = board
        self.initial_black = initial_black or set()
        self.initial_white = initial_white or set()
        self.marked_coordinates = marked_coordinates or {}
        self.first_player = first_player
        if self.first_player == 'black':
            self.second_player = 'white'
        elif self.first_player == 'white':
            self.second_player = 'black'
        else:
            self.first_player = 'black'
            self.second_player = 'white'
        self.tiles = TileSet('tiny')

    def render(self):
        self.image = self.tiles.render(self)

    def show(self):
        self.image.show()

    def save(self, filename, format):
        self.image.save(filename, format)
        
if __name__ == '__main__':
    black_set = set([(4, 4), (4, 5)])
    white_set = set([(3, 2), (7, 3), (8, 3)])
    marked_coordinates = {
        (2, 2): 1,
        (1, 2): 2,
        (2, 3): 3,
        (3, 1): 4,
    } 
    board = Board(
        (9, 9), 
    )
    Diagram(board=board, 
            initial_black=black_set, 
            initial_white=white_set, 
            sequence=sequence).draw()



#!/usr/bin/env python

from setuptools import setup

setup(
    name='rstgo',
    version='0.2.0',
    description='A package to render go diagrams and embed them in reStructuredText documents',
    long_description=open('README.txt').read(),
    author='J. Cliff Dyer',
    author_email='jcd@sdf.lonestar.org',
    url='http://bitbucket.org/cliff/rstgo',
    packages=['rstgo'],
    package_data={'rstgo': ['resources/*/*.png', 'examples/*']},
    scripts=['bin/rst2html+go.py'],
    license='LICENSE.txt',
    install_requires=[
        'docutils',
        'PIL',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Documentation',
        'Topic :: Games/Entertainment :: Board Games',
        'Topic :: Games/Entertainment :: Turn Based Strategy',
    ],
)
